import { Test, TestingModule } from '@nestjs/testing';
import { Messager } from '../../adapters/secondaries/messager';
import { MatchUseCase } from '../../domain/usecases/match.usecase';
import { MatchRequest } from '../../domain/dtos/match.request';
import { MatchQuery } from '../../queries/match.query';
import { AdRepository } from '../../adapters/secondaries/ad.repository';
import { AutomapperModule } from '@automapper/nestjs';
import { classes } from '@automapper/classes';
import { IDefaultParams } from '../../domain/interfaces/default-params.interface';

const mockAdRepository = {};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

const defaultParams: IDefaultParams = {
  DEFAULT_IDENTIFIER: 0,
  MARGIN_DURATION: 900,
  VALIDITY_DURATION: 365,
};

describe('MatchUseCase', () => {
  let matchUseCase: MatchUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],
      providers: [
        {
          provide: AdRepository,
          useValue: mockAdRepository,
        },
        {
          provide: Messager,
          useValue: mockMessager,
        },
        MatchUseCase,
      ],
    }).compile();

    matchUseCase = module.get<MatchUseCase>(MatchUseCase);
  });

  it('should be defined', () => {
    expect(matchUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should return matches', async () => {
      const matchRequest: MatchRequest = new MatchRequest();
      matchRequest.waypoints = [
        {
          lon: 1.093912,
          lat: 49.440041,
        },
        {
          lat: 50.630992,
          lon: 3.045432,
        },
      ];
      matchRequest.departure = '2023-04-01 12:23:00';
      const matches = await matchUseCase.execute(
        new MatchQuery(matchRequest, defaultParams),
      );
      expect(matches.total).toBe(1);
    });
  });
});
